/*
 * $ Copyright Cypress Semiconductor $
 */

/** @file
 *
 * Hibernation test
 *
 */

#include "wiced.h"

/******************************************************
 *                      Macros
 ******************************************************/

#define HIBERNATION_PERIOD_MS 10000
#define PLATFORM_HIB_ENABLE
/******************************************************
 *                    Constants
 ******************************************************/

/******************************************************
 *                   Enumerations
 ******************************************************/

/******************************************************
 *                 Type Definitions
 ******************************************************/

/******************************************************
 *                    Structures
 ******************************************************/

/******************************************************
 *               Function Declarations
 ******************************************************/

/******************************************************
 *               Variable Definitions
 ******************************************************/

/******************************************************
 *               Function Definitions
 ******************************************************/

void application_start( )
{
    uint32_t          freq;
    uint32_t          max_ticks;
    uint32_t          ticks_to_wakeup;
    platform_result_t result = PLATFORM_BADARG;

    printf( "\n\nHibernation test application\n" );

    host_rtos_delay_milliseconds( 5000 );

    freq      = platform_hibernation_get_clock_freq( );
    max_ticks = platform_hibernation_get_max_ticks( );

    printf( "Frequency %u maximum ticks 0x%x\n", (unsigned)freq, (unsigned)max_ticks );

    if ( platform_hibernation_is_returned_from( ) )
    {
        uint32_t spent_ticks = platform_hibernation_get_ticks_spent( );
        uint32_t spent_ms    = 1000 * spent_ticks / freq;

        printf( "Returned from hibernation: spent there %u ticks, %u ms\n", (unsigned)spent_ticks, (unsigned)spent_ms );
    }
    else
    {
        printf( "Normal boot\n" );
    }

    host_rtos_delay_milliseconds( 1000 );

    printf( "After short sleep will hibernate for %u ms\n", (unsigned)HIBERNATION_PERIOD_MS );

    host_rtos_delay_milliseconds( 1000 );

    ticks_to_wakeup = freq * HIBERNATION_PERIOD_MS / 1000;
    if ( ticks_to_wakeup > max_ticks )
    {
        printf( "Scheduled ticks number %u is too big\n", (unsigned)ticks_to_wakeup );
    }
    else
    {
        result = platform_hibernation_start( ticks_to_wakeup );
    }

    while ( 1 )
    {
        printf( "Hibernation failure: %d\n", (int)result );
        host_rtos_delay_milliseconds( 1000 );
    }
}
